@@ -4,13 +4,14 @@ from __future__ import unicode_literals, print_function
 import os
 import tempfile
 import builtins
+import itertools
 from tempfile import TemporaryDirectory
 from xonsh.tools import ON_WINDOWS
 
 import pytest
 
 from xonsh.commands_cache import CommandsCache
-from xonsh.environ import Env, load_static_config, locate_binary
+from xonsh.environ import Env, load_static_config, locate_binary, DEFAULT_ENSURERS, DEFAULT_VALUES
 
 from tools import skip_if_on_unix
 
@@ -140,3 +141,95 @@ def test_locate_binary_on_windows(xonsh_builtins):
         assert locate_binary('file2') == os.path.join(tmpdir, 'FILE2.BAT')
         assert locate_binary('file2.bat') == os.path.join(tmpdir, 'FILE2.BAT')
         assert locate_binary('file3') is None
+
+
+def test_event_on_envvar_change(xonsh_builtins):
+    env = Env(TEST=0)
+    xonsh_builtins.__xonsh_env__ = env
+    share = []
+    # register
+    @xonsh_builtins.events.on_envvar_change
+    def handler(name, oldvalue, newvalue, **kwargs):
+        share.extend((name, oldvalue, newvalue))
+
+    # trigger
+    env['TEST'] = 1
+
+    assert share == ['TEST', 0, 1]
+
+
+def test_event_on_envvar_new(xonsh_builtins):
+    env = Env()
+    xonsh_builtins.__xonsh_env__ = env
+    share = []
+    # register
+    @xonsh_builtins.events.on_envvar_new
+    def handler(name, value, **kwargs):
+        share.extend((name, value))
+
+    # trigger
+    env['TEST'] = 1
+
+    assert share == ['TEST', 1]
+
+def test_event_on_envvar_change_from_none_value(xonsh_builtins):
+    env = Env(TEST=None)
+    xonsh_builtins.__xonsh_env__ = env
+    share = []
+    # register
+    @xonsh_builtins.events.on_envvar_change
+    def handler(name, oldvalue, newvalue, **kwargs):
+        share.extend((name, oldvalue, newvalue))
+
+    # trigger
+    env['TEST'] = 1
+
+    assert share == ['TEST', None, 1]
+
+
+@pytest.mark.parametrize('val', [1, None, True, 'ok'])
+def test_event_on_envvar_change_no_fire_when_value_is_same(val, xonsh_builtins):
+    env = Env(TEST=val)
+    xonsh_builtins.__xonsh_env__ = env
+    share = []
+    # register
+    @xonsh_builtins.events.on_envvar_change
+    def handler(name, oldvalue, newvalue, **kwargs):
+        share.extend((name, oldvalue, newvalue))
+
+
+    # trigger
+    env['TEST'] = val
+
+    assert share == []
+
+
+def test_events_on_envvar_called_in_right_order(xonsh_builtins):
+    env = Env()
+    xonsh_builtins.__xonsh_env__ = env
+    share = []
+    # register
+    @xonsh_builtins.events.on_envvar_new
+    def handler(name, value, **kwargs):
+        share[:] = ['new']
+
+    @xonsh_builtins.events.on_envvar_change
+    def handler(name, oldvalue, newvalue, **kwargs):
+        share[:] = ['change']
+
+    # trigger new
+    env['TEST'] = 1
+
+    assert share == ['new']
+
+    # trigger change
+    env['TEST'] = 2
+
+    assert share == ['change']
+
+
+def test_int_bool_envvars_have_ensurers():
+    bool_ints = [type(envvar) in [bool, int] for envvar in DEFAULT_VALUES.values()]
+    key_mask = set(itertools.compress(DEFAULT_VALUES.keys(), bool_ints))
+    ensurer_keys = set(DEFAULT_ENSURERS.keys())
+    assert len(key_mask.intersection(ensurer_keys)) == len(key_mask)
