@@ -6,16 +6,16 @@ import builtins
 import pytest
 
 from xonsh import imphooks
+from xonsh.execer import Execer
 from xonsh.environ import Env
-from xonsh.built_ins import load_builtins, unload_builtins
+from xonsh.built_ins import unload_builtins
 
-imphooks.install_hook()
+imphooks.install_import_hooks()
 
 
 @pytest.yield_fixture(autouse=True)
-def imp_env(xonsh_execer):
-    """Call `load_builtins` with `xonsh_execer`"""
-    load_builtins(execer=xonsh_execer)
+def imp_env():
+    execer = Execer(unload=False)
     builtins.__xonsh_env__ = Env({'PATH': [], 'PATHEXT': []})
     yield
     unload_builtins()
