@@ -5,9 +5,11 @@ import os
 import pathlib
 import stat
 from tempfile import TemporaryDirectory
+import warnings
 
 import pytest
 
+from xonsh import __version__
 from xonsh.platform import ON_WINDOWS
 from xonsh.lexer import Lexer
 
@@ -24,8 +26,8 @@ from xonsh.tools import (
     is_string_seq, pathsep_to_seq, seq_to_pathsep, is_nonstring_seq_of_strings,
     pathsep_to_upper_seq, seq_to_upper_pathsep, expandvars, is_int_as_str, is_slice_as_str,
     ensure_timestamp, get_portions, is_balanced, subexpr_before_unbalanced,
-    swap_values, get_logical_line, replace_logical_line, check_quotes,
-    )
+    swap_values, get_logical_line, replace_logical_line, check_quotes, deprecated,
+    is_writable_file, balanced_parens)
 from xonsh.environ import Env
 
 from tools import skip_if_on_windows, skip_if_on_unix
@@ -401,6 +403,7 @@ mom"""
  "     --option2 value2 \\\n"
  "     --optionZ valueZ",
 1, "echo --option1 value1      --option2 value2      --optionZ valueZ", 3),
+('"""\n', 0, '"""', 1),
 ]
 
 @pytest.mark.parametrize('src, idx, exp_line, exp_n', LOGICAL_LINE_CASES)
@@ -429,6 +432,7 @@ def test_replace_logical_line(src, idx, exp_line, exp_n):
     ("'y'", True),
     ('b"x"', True),
     ("r'y'", True),
+    ("f'z'", True),
     ('"""hello\nmom"""', True),
 ])
 def test_check_quotes(inp, exp):
@@ -478,6 +482,26 @@ def test_subexpr_before_unbalanced_parens(inp, exp):
     assert exp == obs
 
 
+@pytest.mark.parametrize('line, exp', [
+    ('', True),
+    ('wakka jawaka', True),
+    ('rm *; echo hello world', True),
+    ('()', True),
+    ('f()', True),
+    ('echo * yo ; echo eggs', True),
+    ('(', False),
+    (')', False),
+    ('(cmd;', False),
+    ('cmd;)', False),
+])
+def test_balanced_parens(line, exp):
+    obs = balanced_parens(line, lexer=LEXER)
+    if exp:
+        assert obs
+    else:
+        assert not obs
+
+
 @pytest.mark.parametrize('line, mincol, exp', [
     ('ls && echo a', 0, 4),
     ('ls && echo a', 6, None),
@@ -487,6 +511,7 @@ def test_subexpr_before_unbalanced_parens(inp, exp):
     ('not (ls) && echo a', 0, 8),
     ('bash -c ! export var=42; echo $var', 0, 35),
     ('python -c ! import os; print(os.path.abspath("/"))', 0, 51),
+    ('echo * yo ; echo eggs', 0, 11),
 ])
 def test_find_next_break(line, mincol, exp):
     obs = find_next_break(line, mincol=mincol, lexer=LEXER)
@@ -681,6 +706,19 @@ def test_env_path_to_str(inp, exp):
     assert exp == obs
 
 
+@pytest.mark.parametrize('left, right, exp', [
+    (EnvPath(['/home/wakka']), ['/home/jawaka'], EnvPath(['/home/wakka', '/home/jawaka'])),
+    (['a'], EnvPath(['b']), EnvPath(['a', 'b'])),
+    (EnvPath(['c']), EnvPath(['d']), EnvPath(['c', 'd'])),
+])
+def test_env_path_add(left, right, exp):
+    obs = left + right
+    assert is_env_path(obs)
+    assert exp == obs
+
+
+
+
 # helper
 def expand(path):
     return os.path.expanduser(os.path.expandvars(path))
@@ -1006,7 +1044,8 @@ def test_is_logfile_opt(inp, exp):
         (None, None),
         ('throwback.log', 'throwback.log'),
         skip_if_on_windows(('/dev/null', '/dev/null')),
-        skip_if_on_windows(('/dev/nonexistent_dev', None))
+        skip_if_on_windows(('/dev/nonexistent_dev',
+            '/dev/nonexistent_dev' if is_writable_file('/dev/nonexistent_dev') else None))
     ])
 def test_to_logfile_opt(inp, exp):
     obs = to_logfile_opt(inp)
@@ -1054,7 +1093,7 @@ def test_dynamic_cwd_tuple_to_str(inp, exp):
     ('foo&bar', 'foo^&bar'),
     ('foo$?-/_"\\', 'foo$?-/_^"\\'),
     ('^&<>|', '^^^&^<^>^|'),
-    ('this /?', 'this /.')
+    ('()<>','^(^)^<^>'),
 ])
 def test_escape_windows_cmd_string(st, esc):
     obs = escape_windows_cmd_string(st)
@@ -1247,3 +1286,77 @@ def test_swap_values():
         assert orig['y'] == 43
     assert orig['x'] == 1
     assert 'y' not in orig
+
+
+@pytest.mark.parametrize('arguments, expected_docstring', [
+    ({'deprecated_in': '0.5.10', 'removed_in': '0.6.0'},
+     'my_function has been deprecated in version 0.5.10 and will be removed '
+     'in version 0.6.0'),
+    ({'deprecated_in': '0.5.10'},
+     'my_function has been deprecated in version 0.5.10'),
+    ({'removed_in': '0.6.0'},
+     'my_function has been deprecated and will be removed in version 0.6.0'),
+    ({},
+     'my_function has been deprecated')
+])
+def test_deprecated_docstrings_with_empty_docstring(
+        arguments, expected_docstring):
+    @deprecated(**arguments)
+    def my_function(): pass
+
+    assert my_function.__doc__ == expected_docstring
+
+
+@pytest.mark.parametrize('arguments, expected_docstring', [
+    ({'deprecated_in': '0.5.10', 'removed_in': '0.6.0'},
+     'Does nothing.\n\nmy_function has been deprecated in version 0.5.10 and '
+     'will be removed in version 0.6.0'),
+    ({'deprecated_in': '0.5.10'},
+     'Does nothing.\n\nmy_function has been deprecated in version 0.5.10'),
+    ({'removed_in': '0.6.0'},
+     'Does nothing.\n\nmy_function has been deprecated and will be removed '
+     'in version 0.6.0'),
+    ({},
+     'Does nothing.\n\nmy_function has been deprecated')
+])
+def test_deprecated_docstrings_with_nonempty_docstring(
+        arguments, expected_docstring):
+    @deprecated(**arguments)
+    def my_function():
+        """Does nothing."""
+        pass
+
+    assert my_function.__doc__ == expected_docstring
+
+
+def test_deprecated_warning_raised():
+    @deprecated()
+    def my_function(): pass
+
+    with warnings.catch_warnings(record=True) as warning:
+        warnings.simplefilter('always')
+
+        my_function()
+
+        assert issubclass(warning.pop().category, DeprecationWarning)
+
+
+def test_deprecated_warning_contains_message():
+    @deprecated()
+    def my_function(): pass
+
+    with warnings.catch_warnings(record=True) as warning:
+        warnings.simplefilter('always')
+
+        my_function()
+
+        assert str(warning.pop().message) == 'my_function has been deprecated'
+
+
+@pytest.mark.parametrize('expired_version', ['0.1.0', __version__])
+def test_deprecated_past_expiry_raises_assertion_error(expired_version):
+    @deprecated(removed_in=expired_version)
+    def my_function(): pass
+
+    with pytest.raises(AssertionError):
+        my_function()
