@@ -19,17 +19,19 @@ from xonsh.base_shell import BaseShell
 
 VER_3_4 = (3, 4)
 VER_3_5 = (3, 5)
+VER_3_6 = (3, 6)
 VER_MAJOR_MINOR = sys.version_info[:2]
 VER_FULL = sys.version_info[:3]
 ON_DARWIN = (platform.system() == 'Darwin')
 ON_WINDOWS = (platform.system() == 'Windows')
 ON_CONDA = True in [conda in pytest.__file__.lower() for conda
-                    in ['anaconda', 'miniconda']]
+                    in ['conda', 'anaconda', 'miniconda']]
 ON_TRAVIS = 'TRAVIS' in os.environ and 'CI' in os.environ
 TEST_DIR = os.path.dirname(__file__)
 
 # pytest skip decorators
 skip_if_py34 = pytest.mark.skipif(VER_MAJOR_MINOR < VER_3_5, reason="Py3.5+ only test")
+skip_if_lt_py36 = pytest.mark.skipif(VER_MAJOR_MINOR < VER_3_6, reason="Py3.6+ only test")
 
 skip_if_on_conda = pytest.mark.skipif(ON_CONDA,
                         reason="Conda and virtualenv _really_ hate each other")
