@@ -31,6 +31,13 @@ if PYTHON_VERSION_INFO >= (3, 5, 0):
 else:
     MatMult = AsyncFunctionDef = AsyncWith = AsyncFor = Await = None
 
+if PYTHON_VERSION_INFO >= (3, 6, 0):
+    # pylint: disable=unused-import
+    # pylint: disable=no-name-in-module
+    from ast import JoinedStr, FormattedValue
+else:
+    JoinedStr = FormattedValue = None
+
 STATEMENTS = (FunctionDef, ClassDef, Return, Delete, Assign, AugAssign, For,
               While, If, With, Raise, Try, Assert, Import, ImportFrom, Global,
               Nonlocal, Expr, Pass, Break, Continue)
@@ -52,7 +59,7 @@ def leftmostname(node):
         rtn = leftmostname(node.values[0])
     elif isinstance(node, Assign):
         rtn = leftmostname(node.targets[0])
-    elif isinstance(node, (Str, Bytes)):
+    elif isinstance(node, (Str, Bytes, JoinedStr)):
         # handles case of "./my executable"
         rtn = leftmostname(node.s)
     elif isinstance(node, Tuple) and len(node.elts) > 0:
@@ -160,7 +167,7 @@ def xonsh_call(name, args, lineno=None, col=None):
 
 
 def isdescendable(node):
-    """Deteremines whether or not a node is worth visiting. Currently only
+    """Determines whether or not a node is worth visiting. Currently only
     UnaryOp and BoolOp nodes are visited.
     """
     return isinstance(node, (UnaryOp, BoolOp))
@@ -177,7 +184,7 @@ class CtxAwareTransformer(NodeTransformer):
         """Parameters
         ----------
         parser : xonsh.Parser
-            A parse instance to try to parse suprocess statements with.
+            A parse instance to try to parse subprocess statements with.
         """
         super(CtxAwareTransformer, self).__init__()
         self.parser = parser
@@ -250,6 +257,8 @@ class CtxAwareTransformer(NodeTransformer):
                                          lexer=self.parser.lexer)
             elif nlogical > 1:
                 maxcol = None
+            elif maxcol < len(line) and line[maxcol] == ';':
+                pass
             else:
                 maxcol += 1
         spline = subproc_toks(line, mincol=mincol, maxcol=maxcol,
