@@ -7,6 +7,7 @@ import xonsh.platform as xp
 from xonsh.completers.tools import get_filter_function
 
 SKIP_TOKENS = {'sudo', 'time', 'timeit', 'which', 'showcmd', 'man'}
+END_PROC_TOKENS = {'|', '||', '&&', 'and', 'or'}
 
 
 def complete_command(cmd, line, start, end, ctx):
@@ -33,23 +34,28 @@ def complete_skipper(cmd, line, start, end, ctx):
     Skip over several tokens (e.g., sudo) and complete based on the rest of the
     line.
     """
-    res = line.split(' ', 1)
-    if len(res) == 2:
-        first, rest = res
-    else:
-        first = res[0]
-        rest = ''
-    if first in SKIP_TOKENS:
-        comp = builtins.__xonsh_shell__.shell.completer
-        res = rest.split(' ', 1)
-        if len(res) == 1:
-            comp_func = complete_command
-        else:
-            comp_func = comp.complete
-        return comp_func(cmd,
-                         rest,
-                         start - len(first) - 1,
-                         end - len(first) - 1,
-                         ctx)
-    else:
+    parts = line.split(' ')
+    skip_part_num = 0
+    for i, s in enumerate(parts):
+        if s in END_PROC_TOKENS:
+            skip_part_num = i + 1
+    while len(parts) > skip_part_num:
+        if parts[skip_part_num] not in SKIP_TOKENS:
+            break
+        skip_part_num += 1
+
+    if skip_part_num == 0:
         return set()
+
+    if len(parts) == skip_part_num + 1:
+        comp_func = complete_command
+    else:
+        comp = builtins.__xonsh_shell__.shell.completer
+        comp_func = comp.complete
+
+    skip_len = len(' '.join(line[:skip_part_num])) + 1
+    return comp_func(cmd,
+                     ' '.join(parts[skip_part_num:]),
+                     start - skip_len,
+                     end - skip_len,
+                     ctx)
