@@ -47,7 +47,7 @@ import xonsh.prompt.base as prompt
 events.doc('on_envvar_new', """
 on_envvar_new(name: str, value: Any) -> None
 
-Fires after a new enviroment variable is created.
+Fires after a new environment variable is created.
 Note: Setting envvars inside the handler might
 cause a recursion until the limit.
 """)
@@ -56,7 +56,7 @@ cause a recursion until the limit.
 events.doc('on_envvar_change', """
 on_envvar_change(name: str, oldvalue: Any, newvalue: Any) -> None
 
-Fires after an enviroment variable is changed.
+Fires after an environment variable is changed.
 Note: Setting envvars inside the handler might
 cause a recursion until the limit.
 """)
@@ -120,6 +120,7 @@ def DEFAULT_ENSURERS():
     'AUTO_CD': (is_bool, to_bool, bool_to_str),
     'AUTO_PUSHD': (is_bool, to_bool, bool_to_str),
     'AUTO_SUGGEST': (is_bool, to_bool, bool_to_str),
+    'AUTO_SUGGEST_IN_COMPLETIONS': (is_bool, to_bool, bool_to_str),
     'BASH_COMPLETIONS': (is_env_path, str_to_env_path, env_path_to_str),
     'CASE_SENSITIVE_COMPLETIONS': (is_bool, to_bool, bool_to_str),
     re.compile('\w*DIRS$'): (is_env_path, str_to_env_path, env_path_to_str),
@@ -179,6 +180,7 @@ def DEFAULT_ENSURERS():
     'VIRTUAL_ENV': (is_string, ensure_string, ensure_string),
     'WIN_UNICODE_CONSOLE': (always_false, setup_win_unicode_console, bool_to_str),
     'XONSHRC': (is_env_path, str_to_env_path, env_path_to_str),
+    'XONSH_APPEND_NEWLINE': (is_bool, to_bool, bool_to_str),
     'XONSH_AUTOPAIR': (is_bool, to_bool, bool_to_str),
     'XONSH_CACHE_SCRIPTS': (is_bool, to_bool, bool_to_str),
     'XONSH_CACHE_EVERYTHING': (is_bool, to_bool, bool_to_str),
@@ -264,6 +266,7 @@ def DEFAULT_VALUES():
         'AUTO_CD': False,
         'AUTO_PUSHD': False,
         'AUTO_SUGGEST': True,
+        'AUTO_SUGGEST_IN_COMPLETIONS': True,
         'BASH_COMPLETIONS': BASH_COMPLETIONS_DEFAULT,
         'CASE_SENSITIVE_COMPLETIONS': ON_LINUX,
         'CDPATH': (),
@@ -325,6 +328,7 @@ def DEFAULT_VALUES():
                                                          'share')),
         'XONSHCONFIG': xonshconfig,
         'XONSHRC': default_xonshrc,
+        'XONSH_APPEND_NEWLINE': False,
         'XONSH_AUTOPAIR': False,
         'XONSH_CACHE_SCRIPTS': True,
         'XONSH_CACHE_EVERYTHING': False,
@@ -392,6 +396,10 @@ def DEFAULT_DOCS():
         'Enable automatic command suggestions based on history, like in the fish '
         'shell.\n\nPressing the right arrow key inserts the currently '
         'displayed suggestion. Only usable with ``$SHELL_TYPE=prompt_toolkit.``'),
+    'AUTO_SUGGEST_IN_COMPLETIONS': VarDocs(
+        'Places the auto-suggest result as the first option in the completions. '
+        'This enables you to tab complete the auto-suggestion.'
+        ),
     'BASH_COMPLETIONS': VarDocs(
         'This is a list (or tuple) of strings that specifies where the '
         '``bash_completion`` script may be found. '
@@ -521,7 +529,7 @@ def DEFAULT_DOCS():
                       configurable=False),
     'PATH': VarDocs(
         'List of strings representing where to look for executables.'),
-    'PATHEXT': VarDocs('Sequence of extention strings (eg, ``.EXE``) for '
+    'PATHEXT': VarDocs('Sequence of extension strings (eg, ``.EXE``) for '
                        'filtering valid executables by. Each element must be '
                        'uppercase.'),
     'PRETTY_PRINT_RESULTS': VarDocs(
@@ -545,12 +553,12 @@ def DEFAULT_DOCS():
         'The error that is raised is a ``subprocess.CalledProcessError``.'),
     'RIGHT_PROMPT': VarDocs(
         'Template string for right-aligned text '
-        'at the prompt. This may be parameterized in the same way as '
+        'at the prompt. This may be parametrized in the same way as '
         'the ``$PROMPT`` variable. Currently, this is only available in the '
         'prompt-toolkit shell.'),
     'BOTTOM_TOOLBAR': VarDocs(
         'Template string for the bottom toolbar. '
-        'This may be parameterized in the same way as '
+        'This may be parametrized in the same way as '
         'the ``$PROMPT`` variable. Currently, this is only available in the '
         'prompt-toolkit shell.'),
     'SHELL_TYPE': VarDocs(
@@ -580,7 +588,7 @@ def DEFAULT_DOCS():
         'command will be offered as a suggestion.  Also used for "fuzzy" '
         'tab completion of paths.'),
     'SUPPRESS_BRANCH_TIMEOUT_MESSAGE': VarDocs(
-        'Whether or not to supress branch timeout warning messages.'),
+        'Whether or not to suppress branch timeout warning messages.'),
     'TERM': VarDocs(
         'TERM is sometimes set by the terminal emulator. This is used (when '
         "valid) to determine whether or not to set the title. Users shouldn't "
@@ -645,6 +653,9 @@ def DEFAULT_DOCS():
             "On Linux & Mac OSX: ``['/etc/xonshrc', '~/.xonshrc']``\n"
             "\nOn Windows: "
             "``['%ALLUSERSPROFILE%\\\\xonsh\\\\xonshrc', '~/.xonshrc']``")),
+    'XONSH_APPEND_NEWLINE': VarDocs(
+        'Append new line when a partial line is preserved in output.'
+    ),
     'XONSH_AUTOPAIR': VarDocs(
         'Whether Xonsh will auto-insert matching parentheses, brackets, and '
         'quotes. Only available under the prompt-toolkit shell.'
@@ -664,7 +675,7 @@ def DEFAULT_DOCS():
     'XONSH_DEBUG': VarDocs(
         'Sets the xonsh debugging level. This may be an integer or a boolean. '
         'Setting this variable prior to stating xonsh to ``1`` or ``True`` '
-        'will supress amalgamated imports. Setting it to ``2`` will get some '
+        'will suppress amalgamated imports. Setting it to ``2`` will get some '
         'basic information like input transformation, command replacement. '
         'With ``3`` or a higher number will make more debugging information '
         'presented, like PLY parsing messages.',
@@ -720,7 +731,7 @@ def DEFAULT_DOCS():
         '``True`` if xonsh is running as a login shell, and ``False`` otherwise.',
         configurable=False),
     'XONSH_PROC_FREQUENCY': VarDocs(
-        'The process frquency is the time that '
+        'The process frequency is the time that '
         'xonsh process threads sleep for while running command pipelines. '
         'The value has units of seconds [s].'),
     'XONSH_SHOW_TRACEBACK': VarDocs(
@@ -772,7 +783,7 @@ class Env(cabc.MutableMapping):
 
     * PATH: any variable whose name ends in PATH is a list of strings.
     * XONSH_HISTORY_SIZE: this variable is an (int | float, str) tuple.
-    * LC_* (locale categories): locale catergory names get/set the Python
+    * LC_* (locale categories): locale category names get/set the Python
       locale via locale.getlocale() and locale.setlocale() functions.
 
     An Env instance may be converted to an untyped version suitable for
@@ -831,7 +842,7 @@ class Env(cabc.MutableMapping):
 
     def replace_env(self):
         """Replaces the contents of os_environ with a detyped version
-        of the xonsh environement.
+        of the xonsh environment.
         """
         if self._orig_env is None:
             self._orig_env = dict(os_environ)
@@ -840,7 +851,7 @@ class Env(cabc.MutableMapping):
 
     def undo_replace_env(self):
         """Replaces the contents of os_environ with a detyped version
-        of the xonsh environement.
+        of the xonsh environment.
         """
         if self._orig_env is not None:
             os_environ.clear()
@@ -874,7 +885,7 @@ class Env(cabc.MutableMapping):
         return vd
 
     def help(self, key):
-        """Get information about a specific enviroment variable."""
+        """Get information about a specific environment variable."""
         vardocs = self.get_docs(key)
         width = min(79, os.get_terminal_size()[0])
         docstr = '\n'.join(textwrap.wrap(vardocs.docstr, width=width))
