@@ -170,9 +170,9 @@ def parser():
                    choices=('readline', 'prompt_toolkit', 'best', 'random'),
                    default=None)
     p.add_argument('--timings',
-                   help='Prints timing infomation before the prompt is shown. '
-                        'This is usefull to track down perfomance issues '
-                        'and investigate startup times.',
+                   help='Prints timing information before the prompt is shown. '
+                        'This is useful while tracking down performance issues '
+                        'and investigating startup times.',
                    dest='timings',
                    action='store_true',
                    default=None)
@@ -218,9 +218,9 @@ class XonshMode(enum.Enum):
     interactive = 3
 
 
-def start_services(shell_kwargs):
+def start_services(shell_kwargs, args):
     """Starts up the essential services in the proper order.
-    This returns the envrionment instance as a convenience.
+    This returns the environment instance as a convenience.
     """
     install_import_hooks()
     # create execer, which loads builtins
@@ -236,6 +236,9 @@ def start_services(shell_kwargs):
     env = builtins.__xonsh_env__
     rc = shell_kwargs.get('rc', None)
     rc = env.get('XONSHRC') if rc is None else rc
+    if args.mode != XonshMode.interactive and not args.force_interactive:
+        #  Don't load xonshrc if not interactive shell
+        rc = None
     events.on_pre_rc.fire()
     xonshrc_context(rcfiles=rc, execer=execer, ctx=ctx, env=env, login=login)
     events.on_post_rc.fire()
@@ -246,7 +249,7 @@ def start_services(shell_kwargs):
 
 
 def premain(argv=None):
-    """Setup for main xonsh entry point, returns parsed arguments."""
+    """Setup for main xonsh entry point. Returns parsed arguments."""
     if argv is None:
         argv = sys.argv[1:]
     setup_timings()
@@ -288,11 +291,11 @@ def premain(argv=None):
         args.mode = XonshMode.interactive
         shell_kwargs['completer'] = True
         shell_kwargs['login'] = True
-    env = start_services(shell_kwargs)
+    env = start_services(shell_kwargs, args)
     env['XONSH_LOGIN'] = shell_kwargs['login']
     if args.defines is not None:
         env.update([x.split('=', 1) for x in args.defines])
-    env['XONSH_INTERACTIVE'] = args.force_interactive
+    env['XONSH_INTERACTIVE'] = args.force_interactive or (args.mode == XonshMode.interactive)
     if ON_WINDOWS:
         setup_win_unicode_console(env.get('WIN_UNICODE_CONSOLE', True))
     return args
