@@ -5,6 +5,7 @@ import re
 import time
 import textwrap
 from threading import Thread
+from ast import parse as pyparse
 from collections import Iterable, Sequence, Mapping
 
 try:
@@ -189,8 +190,8 @@ def empty_list_if_newline(x):
 
 
 def lopen_loc(x):
-    """Extracts the line and column number for a node that may have anb opening
-    parenthesis, brace, or braket.
+    """Extracts the line and column number for a node that may have an opening
+    parenthesis, brace, or bracket.
     """
     lineno = x._lopen_lineno if hasattr(x, '_lopen_lineno') else x.lineno
     col = x._lopen_col if hasattr(x, '_lopen_col') else x.col_offset
@@ -390,7 +391,7 @@ class BaseParser(object):
         setattr(self.__class__, listfunc.__name__, listfunc)
 
     def _tok_rule(self, rulename):
-        """For a rule name, creates a rule that retuns the corresponding token.
+        """For a rule name, creates a rule that returns the corresponding token.
         '_tok' is appended to the rule name.
         """
 
@@ -468,6 +469,8 @@ class BaseParser(object):
         else:
             col = loc.column + 1
             lines = self.lines
+            if loc.lineno == 0:
+                loc.lineno = len(lines)
             i = loc.lineno - 1
             if 0 <= i < len(lines):
                 err_line = lines[i].rstrip()
@@ -1437,7 +1440,7 @@ class BaseParser(object):
         self.p_nodedent_base.__func__.__doc__ = doc
 
     def p_nodedent_base(self, p):
-        # see above attachament function
+        # see above attachment function
         pass
 
     def p_nodedent_any(self, p):
@@ -1471,7 +1474,7 @@ class BaseParser(object):
         self.p_nonewline_base.__func__.__doc__ = doc
 
     def p_nonewline_base(self, p):
-        # see above attachament function
+        # see above attachment function
         pass
 
     def p_nonewline_any(self, p):
@@ -1979,7 +1982,7 @@ class BaseParser(object):
         self.p_nocloser_base.__func__.__doc__ = doc
 
     def p_nocloser_base(self, p):
-        # see above attachament function
+        # see above attachment function
         pass
 
     def p_nocloser_any(self, p):
@@ -2000,6 +2003,10 @@ class BaseParser(object):
                         col_offset=p1.lexpos)
             p[0] = xonsh_call('__xonsh_path_literal__', [s],
                               lineno=p1.lineno, col=p1.lexpos)
+        elif 'f' in prefix or 'F' in prefix:
+            s = pyparse(p1.value).body[0].value
+            s = ast.increment_lineno(s, p1.lineno - 1)
+            p[0] = s
         else:
             s = ast.literal_eval(p1.value)
             is_bytes = 'b' in prefix or 'B' in prefix
@@ -2017,7 +2024,7 @@ class BaseParser(object):
     def p_number(self, p):
         """number : number_tok"""
         p1 = p[1]
-        p[0] = ast.Num(n=ast.literal_eval(p1.value), lineno=p1.lineno,
+        p[0] = ast.Num(n=ast.literal_eval(p1.value.replace('_', '')), lineno=p1.lineno,
                        col_offset=p1.lexpos)
 
     def p_testlist_comp_comp(self, p):
@@ -2097,14 +2104,14 @@ class BaseParser(object):
 
     # The following grammar rules are no-ops because we don't need to glue the
     # source code back together piece-by-piece. Instead, we simply look for
-    # top-level commas and record their positions. With these positions and
-    # the bounding parantheses !() positions we can use the source_slice()
-    # method. This does a much better job of capturing exactly the source code
-    # that was provided. The tokenizer & lexer can be a little lossy, especially
-    # with respect to whitespace.
+    # top-level commas and record their positions. With these positions and the
+    # respective positions of the bounding parentheses, we can use the
+    # source_slice() method. This does a much better job of capturing exactly
+    # the source code that was provided. The tokenizer & lexer can be a little
+    # lossy, especially with respect to whitespace.
 
     def p_nocomma_tok(self, p):
-        # see attachement function above for docstring
+        # see attachment function above for docstring
         pass
 
     def p_any_raw_tok(self, p):
@@ -2705,6 +2712,14 @@ class BaseParser(object):
     # Helpers
     #
 
+    def p_test_comma_combine(self, p):
+        """test_comma_list : test comma_test_list
+                           | test comma_test_list COMMA
+        """
+        p2 = p[2]
+        p2.insert(0, p[1])
+        p[0] = p2
+
     def p_empty(self, p):
         'empty : '
         p[0] = None
