@@ -29,8 +29,13 @@ from xonsh.tools import (redirect_stdout, redirect_stderr, print_exception,
                          XonshCalledProcessError, findfirst, on_main_thread,
                          XonshError, format_std_prepost)
 from xonsh.lazyasd import lazyobject, LazyObject
-from xonsh.jobs import wait_for_active_job
+from xonsh.jobs import wait_for_active_job, give_terminal_to, _continue
 from xonsh.lazyimps import fcntl, termios, _winapi, msvcrt, winutils
+# these decorators are imported for users back-compatible
+from xonsh.tools import unthreadable, uncapturable  # NOQA
+
+# foreground has be deprecated
+foreground = unthreadable
 
 
 @lazyobject
@@ -215,8 +220,9 @@ class NonBlockingFDReader(QueueReader):
 
 def populate_buffer(reader, fd, buffer, chunksize):
     """Reads bytes from the file descriptor and copies them into a buffer.
-    The reads happend in parallel, using pread(), and is thus only
-    availabe on posix. If the read fails for any reason, the reader is
+
+    The reads happen in parallel using the pread() syscall; which is only
+    available on POSIX systems. If the read fails for any reason, the reader is
     flagged as closed.
     """
     offset = 0
@@ -275,7 +281,7 @@ def _expand_console_buffer(cols, max_offset, expandsize, orig_posize, fd):
 
 def populate_console(reader, fd, buffer, chunksize, queue, expandsize=None):
     """Reads bytes from the file descriptor and puts lines into the queue.
-    The reads happend in parallel,
+    The reads happened in parallel,
     using xonsh.winutils.read_console_output_character(),
     and is thus only available on windows. If the read fails for any reason,
     the reader is flagged as closed.
@@ -295,23 +301,23 @@ def populate_console(reader, fd, buffer, chunksize, queue, expandsize=None):
     # These chunked reads basically need to happen like this because,
     #   a. The default buffer size is HUGE for the console (90k lines x 120 cols)
     #      as so we can't just read in everything at the end and see what we
-    #      care about without a noticible performance hit.
+    #      care about without a noticeable performance hit.
     #   b. Even with this huge size, it is still possible to write more lines than
     #      this, so we should scroll along with the console.
-    # Unfortnately, because we do not have control over the terminal emulator,
-    # It is not possible to compute how far back we should set the begining
+    # Unfortunately, because we do not have control over the terminal emulator,
+    # It is not possible to compute how far back we should set the beginning
     # read position because we don't know how many characters have been popped
     # off the top of the buffer. If we did somehow know this number we could do
     # something like the following:
     #
     #    new_offset = (y*cols) + x
     #    if new_offset == max_offset:
-    #        new_offset -= scolled_offset
+    #        new_offset -= scrolled_offset
     #        x = new_offset%cols
     #        y = new_offset//cols
     #        continue
     #
-    # So this method is imperfect and only works as long as the sceen has
+    # So this method is imperfect and only works as long as the screen has
     # room to expand to.  Thus the trick here is to expand the screen size
     # when we get close enough to the end of the screen. There remain some
     # async issues related to not being able to set the cursor position.
@@ -327,11 +333,11 @@ def populate_console(reader, fd, buffer, chunksize, queue, expandsize=None):
     # I believe that there is a bug in PTK that if we reset the
     # cursor position, the cursor on the next prompt is accidentally on
     # the next line.  If this is fixed, uncomment the following line.
-    #if max_offset < offset + expandsize:
-    #    rows, max_offset, orig_posize = _expand_console_buffer(
+    # if max_offset < offset + expandsize:
+    #     rows, max_offset, orig_posize = _expand_console_buffer(
     #                                        cols, max_offset, expandsize,
     #                                        orig_posize, fd)
-    #    winutils.set_console_cursor_position(x, y, fd=fd)
+    #     winutils.set_console_cursor_position(x, y, fd=fd)
     while True:
         posize = winutils.get_position_size(fd)
         offset = (cols*y) + x
@@ -483,7 +489,7 @@ def safe_flush(handle):
 
 
 def still_writable(fd):
-    """Determines whether a file descriptior is still writable by trying to
+    """Determines whether a file descriptor is still writable by trying to
     write an empty string and seeing if it fails.
     """
     try:
@@ -544,11 +550,17 @@ class PopenThread(threading.Thread):
         # start up process
         if ON_WINDOWS and stdout is not None:
             os.set_handle_inheritable(stdout.fileno(), False)
-        self.proc = proc = subprocess.Popen(*args,
-                                            stdin=stdin,
-                                            stdout=stdout,
-                                            stderr=stderr,
-                                            **kwargs)
+
+        try:
+            self.proc = proc = subprocess.Popen(*args,
+                                                stdin=stdin,
+                                                stdout=stdout,
+                                                stderr=stderr,
+                                                **kwargs)
+        except Exception:
+            self._clean_up()
+            raise
+
         self.pid = proc.pid
         self.universal_newlines = uninew = proc.universal_newlines
         if uninew:
@@ -653,7 +665,7 @@ class PopenThread(threading.Thread):
 
     def _read_write(self, reader, writer, stdbuf):
         """Reads a chunk of bytes from a buffer and write into memory or back
-        down to the standard buffer, as approriate. Returns the number of
+        down to the standard buffer, as appropriate. Returns the number of
         successful reads.
         """
         if reader is None:
@@ -685,7 +697,7 @@ class PopenThread(threading.Thread):
             self._alt_mode_writer(chunk[:i], membuf, stdbuf)
             # switch modes
             # write the flag itself the current mode where alt mode is on
-            # so that it is streamed to the termial ASAP.
+            # so that it is streamed to the terminal ASAP.
             # this is needed for terminal emulators to find the correct
             # positions before and after alt mode.
             alt_mode = (flag in START_ALTERNATE_MODE)
@@ -760,7 +772,7 @@ class PopenThread(threading.Thread):
             self.old_int_handler = None
         if frame is not None:
             self._disable_cbreak_stdin()
-            if old is not None:
+            if old is not None and old is not self._signal_int:
                 old(signal.SIGINT, frame)
 
     #
@@ -833,7 +845,10 @@ class PopenThread(threading.Thread):
         new[LFLAG] |= termios.ECHO | termios.ICANON
         new[CC][termios.VMIN] = 1
         new[CC][termios.VTIME] = 0
-        termios.tcsetattr(self.stdin_fd, termios.TCSANOW, new)
+        try:
+            termios.tcsetattr(self.stdin_fd, termios.TCSANOW, new)
+        except termios.error:
+            pass
 
     #
     # Dispatch methods
@@ -855,16 +870,24 @@ class PopenThread(threading.Thread):
         if self.old_winch_handler is not None and on_main_thread():
             signal.signal(signal.SIGWINCH, self.old_winch_handler)
             self.old_winch_handler = None
+        self._clean_up()
+        return rtn
+
+    def _clean_up(self):
         self._restore_sigint()
         self._restore_sigtstp()
         self._restore_sigquit()
-        return rtn
 
     @property
     def returncode(self):
         """Process return code."""
         return self.proc.returncode
 
+    @returncode.setter
+    def returncode(self, value):
+        """Process return code."""
+        self.proc.returncode = value
+
     @property
     def signal(self):
         """Process signal, or None."""
@@ -875,6 +898,11 @@ class PopenThread(threading.Thread):
                 s = (-1*rtn, rtn < 0 if ON_WINDOWS else os.WCOREDUMP(rtn))
         return s
 
+    @signal.setter
+    def signal(self, value):
+        """Process signal, or None."""
+        self.proc.signal = value
+
     def send_signal(self, signal):
         """Dispatches to Popen.send_signal()."""
         dt = 0.0
@@ -923,7 +951,7 @@ class Handle(int):
 
 class FileThreadDispatcher:
     """Dispatches to different file handles depending on the
-    current thread. Useful if you want file operation to go to differnt
+    current thread. Useful if you want file operation to go to different
     places for different threads.
     """
 
@@ -933,7 +961,7 @@ class FileThreadDispatcher:
         ----------
         default : file-like or None, optional
             The file handle to write to if a thread cannot be found in
-            the registery. If None, a new in-memory instance.
+            the registry. If None, a new in-memory instance.
 
         Attributes
         ----------
@@ -1084,13 +1112,13 @@ class FileThreadDispatcher:
 
 
 # These should NOT be lazy since they *need* to get the true stdout from the
-# main thread. Also their creation time should be neglibible.
+# main thread. Also their creation time should be negligible.
 STDOUT_DISPATCHER = FileThreadDispatcher(default=sys.stdout)
 STDERR_DISPATCHER = FileThreadDispatcher(default=sys.stderr)
 
 
 def parse_proxy_return(r, stdout, stderr):
-    """Proxies may return a variety of outputs. This hanles them generally.
+    """Proxies may return a variety of outputs. This handles them generally.
 
     Parameters
     ----------
@@ -1158,19 +1186,26 @@ def proxy_four(f, args, stdin, stdout, stderr, spec):
 
 
 PROXIES = (proxy_zero, proxy_one, proxy_two, proxy_three, proxy_four)
+PROXY_KWARG_NAMES = frozenset(['args', 'stdin', 'stdout', 'stderr', 'spec'])
 
 
 def partial_proxy(f):
-    """Dispatches the approriate proxy function based on the number of args."""
-    numargs = len(inspect.signature(f).parameters)
+    """Dispatches the appropriate proxy function based on the number of args."""
+    numargs = 0
+    for name, param in inspect.signature(f).parameters.items():
+        if param.kind == param.POSITIONAL_ONLY or \
+           param.kind == param.POSITIONAL_OR_KEYWORD:
+            numargs += 1
+        elif name in PROXY_KWARG_NAMES and param.kind == param.KEYWORD_ONLY:
+            numargs += 1
     if numargs < 5:
         return functools.partial(PROXIES[numargs], f)
     elif numargs == 5:
         # don't need to partial.
         return f
     else:
-        e = 'Expected proxy with 5 or fewer arguments, not {}'
-        raise XonshError(e.format(numargs))
+        e = 'Expected proxy with 5 or fewer arguments for {}, not {}'
+        raise XonshError(e.format(', '.join(PROXY_KWARG_NAMES), numargs))
 
 
 class ProcProxyThread(threading.Thread):
@@ -1202,7 +1237,8 @@ class ProcProxyThread(threading.Thread):
             set to `None`, then `sys.stderr` is used.
         universal_newlines : bool, optional
             Whether or not to use universal newlines.
-        env : Mapping, optiona            Environment mapping.
+        env : Mapping, optional
+            Environment mapping.
         """
         self.orig_f = f
         self.f = partial_proxy(f)
@@ -1221,6 +1257,7 @@ class ProcProxyThread(threading.Thread):
         self.stdout = stdout
         self.stderr = stderr
         self.env = env or builtins.__xonsh_env__
+        self._interrupted = False
 
         if ON_WINDOWS:
             if self.p2cwrite != -1:
@@ -1248,9 +1285,19 @@ class ProcProxyThread(threading.Thread):
             if universal_newlines:
                 self.stderr = io.TextIOWrapper(self.stderr)
 
+        # Set some signal handles, if we can. Must come before process
+        # is started to prevent deadlock on windows
+        self.old_int_handler = None
+        if on_main_thread():
+            self.old_int_handler = signal.signal(signal.SIGINT,
+                                                 self._signal_int)
+        # start up the proc
         super().__init__()
         self.start()
 
+    def __del__(self):
+        self._restore_sigint()
+
     def run(self):
         """Set up input/output streams and execute the child function in a new
         thread.  This is part of the `threading.Thread` interface and should
@@ -1354,8 +1401,42 @@ class ProcProxyThread(threading.Thread):
     def wait(self, timeout=None):
         """Waits for the process to finish and returns the return code."""
         self.join()
+        self._restore_sigint()
         return self.returncode
 
+    #
+    # SIGINT handler
+    #
+
+    def _signal_int(self, signum, frame):
+        """Signal handler for SIGINT - Ctrl+C may have been pressed."""
+        # Check if we have already been interrupted. This should prevent
+        # the possibility of infinite recursion.
+        if self._interrupted:
+            return
+        self._interrupted = True
+        # close file handles here to stop an processes piped to us.
+        handles = (self.p2cread, self.p2cwrite, self.c2pread, self.c2pwrite,
+                   self.errread, self.errwrite)
+        for handle in handles:
+            safe_fdclose(handle)
+        if self.poll() is not None:
+            self._restore_sigint(frame=frame)
+        if on_main_thread():
+            signal.pthread_kill(threading.get_ident(), signal.SIGINT)
+
+    def _restore_sigint(self, frame=None):
+        old = self.old_int_handler
+        if old is not None:
+            if on_main_thread():
+                signal.signal(signal.SIGINT, old)
+            self.old_int_handler = None
+        if frame is not None:
+            if old is not None and old is not self._signal_int:
+                old(signal.SIGINT, frame)
+        if self._interrupted:
+            self.returncode = 1
+
     # The code below (_get_devnull, _get_handles, and _make_inheritable) comes
     # from subprocess.py in the Python 3.4.2 Standard Library
     def _get_devnull(self):
@@ -1527,7 +1608,7 @@ class ProcProxy(object):
 
     def wait(self, timeout=None):
         """Runs the function and returns the result. Timeout argument only
-        present for API compatability.
+        present for API compatibility.
         """
         if self.f is None:
             return 0
@@ -1539,7 +1620,11 @@ class ProcProxy(object):
         if self.stdin is None:
             stdin = None
         else:
-            stdin = io.TextIOWrapper(self.stdin, encoding=enc, errors=err)
+            if isinstance(self.stdin, int):
+                inbuf = io.open(self.stdin, 'rb', -1)
+            else:
+                inbuf = self.stdin
+            stdin = io.TextIOWrapper(inbuf, encoding=enc, errors=err)
         stdout = self._pick_buf(self.stdout, sys.stdout, enc, err)
         stderr = self._pick_buf(self.stderr, sys.stderr, enc, err)
         # run the actual function
@@ -1578,26 +1663,6 @@ class ProcProxy(object):
         return getattr(self, name)
 
 
-def unthreadable(f):
-    """Decorator that specifies that a callable alias should be run only
-    on the main thread process. This is often needed for debuggers and profilers.
-    """
-    f.__xonsh_threadable__ = False
-    return f
-
-
-foreground = unthreadable
-
-
-def uncapturable(f):
-    """Decorator that specifies that a callable alias should not be run with
-    any capturing. This is often needed if the alias call interactive subprocess,
-    like pagers and text editors.
-    """
-    f.__xonsh_capturable__ = False
-    return f
-
-
 @lazyobject
 def SIGNAL_MESSAGES():
     sm = {
@@ -1634,6 +1699,17 @@ def safe_readable(handle):
     return status
 
 
+def update_fg_process_group(pipeline_group, background):
+    if background:
+        return False
+    if not ON_POSIX:
+        return False
+    env = builtins.__xonsh_env__
+    if not env.get('XONSH_INTERACTIVE'):
+        return False
+    return give_terminal_to(pipeline_group)
+
+
 class CommandPipeline:
     """Represents a subprocess-mode command pipeline."""
 
@@ -1644,18 +1720,12 @@ class CommandPipeline:
 
     nonblocking = (io.BytesIO, NonBlockingFDReader, ConsoleParallelReader)
 
-    def __init__(self, specs, procs, starttime=None, captured=False):
+    def __init__(self, specs):
         """
         Parameters
         ----------
         specs : list of SubprocSpec
-            Process sepcifications
-        procs : list of Popen-like
-            Process objects.
-        starttime : floats or None, optional
-            Start timestamp.
-        captured : bool or str, optional
-            Flag for whether or not the command should be captured.
+            Process specifications
 
         Attributes
         ----------
@@ -1673,18 +1743,38 @@ class CommandPipeline:
             A string of the standard error.
         lines : list of str
             The output lines
+        starttime : floats or None
+            Pipeline start timestamp.
         """
-        self.procs = procs
-        self.proc = procs[-1]
+        self.starttime = None
+        self.ended = False
+        self.procs = []
         self.specs = specs
         self.spec = specs[-1]
-        self.starttime = starttime or time.time()
-        self.captured = captured
-        self.ended = False
+        self.captured = specs[-1].captured
         self.input = self._output = self.errors = self.endtime = None
         self._closed_handle_cache = {}
         self.lines = []
         self._stderr_prefix = self._stderr_postfix = None
+        self.term_pgid = None
+
+        background = self.spec.background
+        pipeline_group = None
+        for spec in specs:
+            if self.starttime is None:
+                self.starttime = time.time()
+            try:
+                proc = spec.run(pipeline_group=pipeline_group)
+            except XonshError:
+                self._return_terminal()
+                raise
+            if proc.pid and pipeline_group is None and not spec.is_proxy and \
+                    self.captured != 'object':
+                pipeline_group = proc.pid
+                if update_fg_process_group(pipeline_group, background):
+                    self.term_pgid = pipeline_group
+            self.procs.append(proc)
+        self.proc = self.procs[-1]
 
     def __repr__(self):
         s = self.__class__.__name__ + '('
@@ -1711,7 +1801,7 @@ class CommandPipeline:
         """Iterates through the last stdout, and returns the lines
         exactly as found.
         """
-        # get approriate handles
+        # get appropriate handles
         spec = self.spec
         proc = self.proc
         timeout = builtins.__xonsh_env__.get('XONSH_PROC_FREQUENCY')
@@ -1724,7 +1814,8 @@ class CommandPipeline:
             stdout = stdout.buffer
         if stdout is not None and not isinstance(stdout, self.nonblocking):
             stdout = NonBlockingFDReader(stdout.fileno(), timeout=timeout)
-        if not stdout or not safe_readable(stdout):
+        if not stdout or self.captured == 'stdout' or not safe_readable(stdout) or \
+                not spec.threadable:
             # we get here if the process is not threadable or the
             # class is the real Popen
             PrevProcCloser(pipeline=self)
@@ -1734,6 +1825,15 @@ class CommandPipeline:
                 self._endtime()
                 if self.captured == 'object':
                     self.end(tee_output=False)
+                elif self.captured == 'hiddenobject' and stdout:
+                    b = stdout.read()
+                    lines = b.splitlines(keepends=True)
+                    yield from lines
+                    self.end(tee_output=False)
+                elif self.captured == 'stdout':
+                    b = stdout.read()
+                    s = self._decode_uninew(b, universal_newlines=True)
+                    self.lines = s.splitlines(keepends=True)
             raise StopIteration
         # get the correct stderr
         stderr = proc.stderr
@@ -1817,7 +1917,8 @@ class CommandPipeline:
 
     def tee_stdout(self):
         """Writes the process stdout to the output variable, line-by-line, and
-        yields each line.
+        yields each line. This may optionally accept lines (in bytes) to iterate
+        over, in which case it does not call iterraw().
         """
         env = builtins.__xonsh_env__
         enc = env.get('XONSH_ENCODING')
@@ -1880,7 +1981,7 @@ class CommandPipeline:
         else:
             self.errors += s
 
-    def _decode_uninew(self, b):
+    def _decode_uninew(self, b, universal_newlines=None):
         """Decode bytes into a str and apply universal newlines as needed."""
         if not b:
             return ''
@@ -1890,7 +1991,7 @@ class CommandPipeline:
                          errors=env.get('XONSH_ENCODING_ERRORS'))
         else:
             s = b
-        if self.spec.universal_newlines:
+        if universal_newlines or self.spec.universal_newlines:
             s = s.replace('\r\n', '\n').replace('\r', '\n')
         return s
 
@@ -1899,11 +2000,20 @@ class CommandPipeline:
     #
 
     def end(self, tee_output=True):
-        """Waits for the command to complete and then runs any closing and
-        cleanup procedures that need to be run.
+        """
+        End the pipeline, return the controlling terminal if needed.
+
+        Main things done in self._end().
         """
         if self.ended:
             return
+        self._end(tee_output=tee_output)
+        self._return_terminal()
+
+    def _end(self, tee_output):
+        """Waits for the command to complete and then runs any closing and
+        cleanup procedures that need to be run.
+        """
         if tee_output:
             for _ in self.tee_stdout():
                 pass
@@ -1918,6 +2028,28 @@ class CommandPipeline:
         self.ended = True
         self._raise_subproc_error()
 
+    def _return_terminal(self):
+        if ON_WINDOWS or not ON_POSIX:
+            return
+        pgid = os.getpgid(0)
+        if self.term_pgid is None or pgid == self.term_pgid:
+            return
+        if give_terminal_to(pgid):  # if gave term succeed
+            self.term_pgid = pgid
+            if hasattr(builtins, '__xonsh_shell__'):
+                # restoring sanity could probably be called whenever we return
+                # control to the shell. But it only seems to matter after a
+                # ^Z event. This *has* to be called after we give the terminal
+                # back to the shell.
+                builtins.__xonsh_shell__.shell.restore_tty_sanity()
+
+    def resume(self, job, tee_output=True):
+        self.ended = False
+        if give_terminal_to(job['pgrp']):
+            self.term_pgid = job['pgrp']
+        _continue(job)
+        self.end(tee_output=tee_output)
+
     def _endtime(self):
         """Sets the closing timestamp if it hasn't been already."""
         if self.endtime is None:
@@ -1965,7 +2097,7 @@ class CommandPipeline:
         self._safe_close(s.captured_stderr)
 
     def _set_input(self):
-        """Sets the input vaiable."""
+        """Sets the input variable."""
         stdin = self.proc.stdin
         if stdin is None or isinstance(stdin, int) or stdin.closed or \
            not stdin.seekable() or not safe_readable(stdin):
@@ -1976,7 +2108,7 @@ class CommandPipeline:
         self.input = self._decode_uninew(input)
 
     def _check_signal(self):
-        """Checks if a signal was recieved and issues a message."""
+        """Checks if a signal was received and issues a message."""
         proc_signal = getattr(self.proc, 'signal', None)
         if proc_signal is None:
             return
@@ -1996,7 +2128,7 @@ class CommandPipeline:
             hist.last_cmd_rtn = self.proc.returncode
 
     def _raise_subproc_error(self):
-        """Raises a subprocess error, if we are suppossed to."""
+        """Raises a subprocess error, if we are supposed to."""
         spec = self.spec
         rtn = self.returncode
         if (not spec.is_proxy and
@@ -2167,7 +2299,7 @@ def pause_call_resume(p, f, *args, **kwargs):
 class PrevProcCloser(threading.Thread):
     """Previous process closer thread for pipelines whose last command
     is itself unthreadable. This makes sure that the pipeline is
-    driven foreward and does not deadlock.
+    driven forward and does not deadlock.
     """
 
     def __init__(self, pipeline):
@@ -2183,7 +2315,7 @@ class PrevProcCloser(threading.Thread):
         self.start()
 
     def run(self):
-        """Runs the closing algorithim."""
+        """Runs the closing algorithm."""
         pipeline = self.pipeline
         check_prev_done = len(pipeline.procs) == 1
         if check_prev_done:
