@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-"""Prompt formatter for simple version control branchs"""
+"""Prompt formatter for simple version control branches"""
 # pylint:disable=no-member, invalid-name
 
 import os
@@ -13,10 +13,11 @@ import xonsh.tools as xt
 
 
 def _get_git_branch(q):
+    denv = builtins.__xonsh_env__.detype()
     try:
         branches = xt.decode_bytes(subprocess.check_output(
-            ['git', 'branch'],
-            stderr=subprocess.DEVNULL
+            ['git', 'branch'], env=denv,
+            stderr=subprocess.DEVNULL,
         )).splitlines()
     except (subprocess.CalledProcessError, OSError, FileNotFoundError):
         q.put(None)
@@ -153,13 +154,15 @@ def current_branch():
 
 def _git_dirty_working_directory(q, include_untracked):
     status = None
+    denv = builtins.__xonsh_env__.detype()
     try:
         cmd = ['git', 'status', '--porcelain']
         if include_untracked:
             cmd.append('--untracked-files=normal')
         else:
             cmd.append('--untracked-files=no')
-        status = subprocess.check_output(cmd, stderr=subprocess.DEVNULL)
+        status = subprocess.check_output(cmd, stderr=subprocess.DEVNULL,
+                                         env=denv)
     except (subprocess.CalledProcessError, OSError, FileNotFoundError):
         q.put(None)
     if status is not None:
@@ -210,9 +213,9 @@ def dirty_working_directory():
     """
     dwd = None
     cmds = builtins.__xonsh_commands_cache__
-    if cmds.lazy_locate_binary('git'):
+    if cmds.lazy_locate_binary('git', ignore_alias=True):
         dwd = git_dirty_working_directory()
-    if cmds.lazy_locate_binary('hg') and dwd is None:
+    if cmds.lazy_locate_binary('hg', ignore_alias=True) and dwd is None:
         dwd = hg_dirty_working_directory()
     return dwd
 
@@ -234,7 +237,7 @@ def branch_color():
 
 def branch_bg_color():
     """Return red if the current branch is dirty, yellow if the dirtiness can
-    not be determined, and green if it clean. These are bacground colors.
+    not be determined, and green if it clean. These are background colors.
     """
     dwd = dirty_working_directory()
     if dwd is None:
