@@ -1,6 +1,5 @@
 # -*- coding: utf-8 -*-
 """The xonsh shell"""
-import os
 import sys
 import random
 import time
@@ -8,12 +7,9 @@ import difflib
 import builtins
 import warnings
 
-from xonsh.xontribs import update_context, prompt_xontrib_install
-from xonsh.environ import xonshrc_context
-from xonsh.execer import Execer
 from xonsh.platform import (best_shell_type, has_prompt_toolkit,
                             ptk_version_is_supported)
-from xonsh.tools import XonshError, to_bool_or_int, print_exception
+from xonsh.tools import XonshError, print_exception
 from xonsh.events import events
 import xonsh.history.main as xhm
 
@@ -22,7 +18,7 @@ events.doc('on_transform_command', """
 on_transform_command(cmd: str) -> str
 
 Fired to request xontribs to transform a command line. Return the transformed
-command, or the same command if no transformaiton occurs. Only done for
+command, or the same command if no transformation occurs. Only done for
 interactive sessions.
 
 This may be fired multiple times per command, with other transformers input or
@@ -39,6 +35,25 @@ events.doc('on_postcommand', """
 on_postcommand(cmd: str, rtn: int, out: str or None, ts: list) -> None
 
 Fires just after a command is executed. The arguments are the same as history.
+
+Parameters:
+
+* ``cmd``: The command that was executed (after transformation)
+* ``rtn``: The result of the command executed (``0`` for success)
+* ``out``: If xonsh stores command output, this is the output
+* ``ts``: Timestamps, in the order of ``[starting, ending]``
+""")
+
+events.doc('on_pre_prompt', """
+on_first_prompt() -> None
+
+Fires just before the prompt is shown
+""")
+
+events.doc('on_post_prompt', """
+on_first_prompt() -> None
+
+Fires just after the prompt returns
 """)
 
 
@@ -57,8 +72,8 @@ def transform_command(src, show_diff=True):
                 break
         i += 1
         if i == limit:
-            print_exception('Modifcations to source input took more than '
-                            'the recursion limit number of interations to '
+            print_exception('Modifications to source input took more than '
+                            'the recursion limit number of iterations to '
                             'converge.')
     debug_level = builtins.__xonsh_env__.get('XONSH_DEBUG')
     if show_diff and debug_level > 1 and src != raw:
@@ -78,11 +93,12 @@ class Shell(object):
     readline version of shell should be used.
     """
 
-    def __init__(self, ctx=None, shell_type=None, config=None, rc=None,
-                 **kwargs):
+    def __init__(self, execer, ctx=None, shell_type=None, **kwargs):
         """
         Parameters
         ----------
+        execer : Execer
+            An execer instance capable of running xonsh code.
         ctx : Mapping, optional
             The execution context for the shell (e.g. the globals namespace).
             If none, this is computed by loading the rc files. If not None,
@@ -91,17 +107,9 @@ class Shell(object):
         shell_type : str, optional
             The shell type to start, such as 'readline', 'prompt_toolkit',
             or 'random'.
-        config : str, optional
-            Path to configuration file.
-        rc : list of str, optional
-            Sequence of paths to run control files.
         """
-        self.login = kwargs.get('login', True)
-        self.stype = shell_type
-        self._init_environ(ctx, config, rc,
-                           kwargs.get('scriptcache', True),
-                           kwargs.get('cacheall', False))
-
+        self.execer = execer
+        self.ctx = {} if ctx is None else ctx
         env = builtins.__xonsh_env__
         # build history backend before creating shell
         builtins.__xonsh_history__ = hist = xhm.construct_history(
@@ -129,7 +137,7 @@ class Shell(object):
                               'supported. Please update prompt-toolkit. Using '
                               'readline instead.')
                 shell_type = 'readline'
-        env['SHELL_TYPE'] = shell_type
+        self.shell_type = env['SHELL_TYPE'] = shell_type
         # actually make the shell
         if shell_type == 'none':
             from xonsh.base_shell import BaseShell as shell_class
@@ -142,33 +150,10 @@ class Shell(object):
                              shell_type))
         self.shell = shell_class(execer=self.execer,
                                  ctx=self.ctx, **kwargs)
-        # allows history garbage colector to start running
+        # allows history garbage collector to start running
         if hist.gc is not None:
             hist.gc.wait_for_shell = False
 
     def __getattr__(self, attr):
         """Delegates calls to appropriate shell instance."""
         return getattr(self.shell, attr)
-
-    def _init_environ(self, ctx, config, rc, scriptcache, cacheall):
-        self.ctx = {} if ctx is None else ctx
-        debug = to_bool_or_int(os.getenv('XONSH_DEBUG', '0'))
-        self.execer = Execer(config=config, login=self.login, xonsh_ctx=self.ctx,
-                             debug_level=debug)
-        self.execer.scriptcache = scriptcache
-        self.execer.cacheall = cacheall
-        if self.stype != 'none' or self.login:
-            # load xontribs from config file
-            names = builtins.__xonsh_config__.get('xontribs', ())
-            for name in names:
-                update_context(name, ctx=self.ctx)
-            if getattr(update_context, 'bad_imports', None):
-                prompt_xontrib_install(update_context.bad_imports)
-                del update_context.bad_imports
-            # load run control files
-            env = builtins.__xonsh_env__
-            rc = env.get('XONSHRC') if rc is None else rc
-            events.on_pre_rc.fire()
-            self.ctx.update(xonshrc_context(rcfiles=rc, execer=self.execer, initial=self.ctx))
-            events.on_post_rc.fire()
-        self.ctx['__name__'] = '__main__'
