@@ -1,22 +1,100 @@
-"""API for Vox, the Python virtual environment manager for xonsh."""
+"""
+API for Vox, the Python virtual environment manager for xonsh.
+
+Vox defines several events related to the life cycle of virtual environments:
+
+* ``vox_on_create(env: str) -> None``
+* ``vox_on_activate(env: str) -> None``
+* ``vox_on_deactivate(env: str) -> None``
+* ``vox_on_delete(env: str) -> None``
+"""
 import os
+import sys
 import venv
 import shutil
 import builtins
 import collections.abc
 
-from xonsh.platform import ON_POSIX, ON_WINDOWS, scandir
+from xonsh.platform import ON_POSIX, ON_WINDOWS
+from xonsh.fs import PathLike, fspath
+
+# This is because builtins aren't globally created during testing.
+# FIXME: Is there a better way?
+from xonsh.events import events
 
 
-VirtualEnvironment = collections.namedtuple('VirtualEnvironment', ['env', 'bin'])
+events.doc('vox_on_create', """
+vox_on_create(env: str) -> None
+
+Fired after an environment is created.
+""")
+
+events.doc('vox_on_activate', """
+vox_on_activate(env: str) -> None
+
+Fired after an environment is activated.
+""")
+
+events.doc('vox_on_deactivate', """
+vox_on_deactivate(env: str) -> None
+
+Fired after an environment is deactivated.
+""")
+
+events.doc('vox_on_delete', """
+vox_on_delete(env: str) -> None
+
+Fired after an environment is deleted (through vox).
+""")
+
+
+VirtualEnvironment = collections.namedtuple('VirtualEnvironment', ['env', 'bin', 'lib', 'inc'])
+
+
+def _subdir_names():
+    """
+    Gets the names of the special dirs in a venv.
+
+    This is not necessarily exhaustive of all the directories that could be in a venv, and there
+    may additional logic to get to useful places.
+    """
+    if ON_WINDOWS:
+        return 'Scripts', 'Lib', 'Include'
+    elif ON_POSIX:
+        return 'bin', 'lib', 'include'
+    else:
+        raise OSError('This OS is not supported.')
+
+
+def _mkvenv(env_dir):
+    """
+    Constructs a VirtualEnvironment based on the given base path.
+
+    This only cares about the platform. No filesystem calls are made.
+    """
+    env_dir = os.path.normpath(env_dir)
+    if ON_WINDOWS:
+        binname = os.path.join(env_dir, 'Scripts')
+        incpath = os.path.join(env_dir, 'Include')
+        libpath = os.path.join(env_dir, 'Lib', 'site-packages')
+    elif ON_POSIX:
+        binname = os.path.join(env_dir, 'bin')
+        incpath = os.path.join(env_dir, 'include')
+        libpath = os.path.join(env_dir, 'lib',
+                               'python%d.%d' % sys.version_info[:2],
+                               'site-packages')
+    else:
+        raise OSError('This OS is not supported.')
+
+    return VirtualEnvironment(env_dir, binname, libpath, incpath)
 
 
 class EnvironmentInUse(Exception):
-    pass
+    """The given environment is currently activated, and the operation cannot be performed."""
 
 
 class NoEnvironmentActive(Exception):
-    pass
+    """No environment is currently activated, and the operation cannot be performed."""
 
 
 class Vox(collections.abc.Mapping):
@@ -55,11 +133,17 @@ class Vox(collections.abc.Mapping):
         """
         # NOTE: clear=True is the same as delete then create.
         # NOTE: upgrade=True is its own method
-        env_path = os.path.join(self.venvdir, name)
+        if isinstance(name, PathLike):
+            env_path = fspath(name)
+        else:
+            env_path = os.path.join(self.venvdir, name)
+        if not self._check_reserved(env_path):
+            raise ValueError("venv can't contain reserved names ({})".format(', '.join(_subdir_names())))
         venv.create(
             env_path,
             system_site_packages=system_site_packages, symlinks=symlinks,
             with_pip=with_pip)
+        events.vox_on_create.fire(name=name)
 
     def upgrade(self, name, *, symlinks=False, with_pip=True):
         """Create a virtual environment in $VIRTUALENV_HOME with python3's ``venv``.
@@ -100,13 +184,8 @@ class Vox(collections.abc.Mapping):
         venv.create(env_path, upgrade=True, **flags)
 
     @staticmethod
-    def _binname():
-        if ON_WINDOWS:
-            return 'Scripts'
-        elif ON_POSIX:
-            return 'bin'
-        else:
-            raise OSError('This OS is not supported.')
+    def _check_reserved(name):
+        return os.path.basename(name) not in _subdir_names()  # FIXME: Check the middle components, too
 
     def __getitem__(self, name):
         """Get information about a virtual environment.
@@ -118,26 +197,50 @@ class Vox(collections.abc.Mapping):
             the current one (throws a KeyError if there isn't one).
         """
         if name is ...:
-            env_path = builtins.__xonsh_env__['VIRTUAL_ENV']
-        elif os.path.isabs(name):
-            env_path = name
+            env_paths = [builtins.__xonsh_env__['VIRTUAL_ENV']]
+        elif isinstance(name, PathLike):
+            env_paths = [fspath(name)]
+        else:
+            if not self._check_reserved(name):
+                # Don't allow a venv that could be a venv special dir
+                raise KeyError()
+
+            env_paths = []
+            if os.path.isdir(name):
+                env_paths += [name]
+            env_paths += [os.path.join(self.venvdir, name)]
+
+        for ep in env_paths:
+            ve = _mkvenv(ep)
+
+            # Actually check if this is an actual venv or just a organizational directory
+            # eg, if 'spam/eggs' is a venv, reject 'spam'
+            if not os.path.exists(ve.bin):
+                continue
+            return ve
         else:
-            env_path = os.path.join(self.venvdir, name)
-        bin_dir = self._binname()
-        bin_path = os.path.join(env_path, bin_dir)
-        # Actually check if this is an actual venv or just a organizational directory
-        # eg, if 'spam/eggs' is a venv, reject 'spam'
-        if not os.path.exists(bin_path):
             raise KeyError()
-        return VirtualEnvironment(env_path, bin_path)
+
+    def __contains__(self, name):
+        # For some reason, MutableMapping seems to do this against iter, which is just silly.
+        try:
+            self[name]
+        except KeyError:
+            return False
+        else:
+            return True
 
     def __iter__(self):
         """List available virtual environments found in $VIRTUALENV_HOME.
         """
-        # FIXME: Handle subdirs--this won't discover eg ``spam/eggs``
-        for x in scandir(self.venvdir):
-            if x.is_dir():
-                yield x.name
+        bin_, lib, inc = _subdir_names()
+        for dirpath, dirnames, _ in os.walk(self.venvdir):
+            if bin_ in dirnames and lib in dirnames:
+                yield dirpath[len(self.venvdir)+1:]  # +1 is to remove the separator
+                # Don't recurse in to the special dirs
+                dirnames.remove(bin_)
+                dirnames.remove(lib)  # This one in particular is likely to be quite large.
+                dirnames.remove(inc)
 
     def __len__(self):
         """Counts known virtual environments, using the same rules as iter().
@@ -159,7 +262,7 @@ class Vox(collections.abc.Mapping):
         env_path = builtins.__xonsh_env__['VIRTUAL_ENV']
         if env_path.startswith(self.venvdir):
             name = env_path[len(self.venvdir):]
-            if name[0] == '/':
+            if name[0] in '/\\':
                 name = name[1:]
             return name
         else:
@@ -175,25 +278,26 @@ class Vox(collections.abc.Mapping):
             Virtual environment name or absolute path.
         """
         env = builtins.__xonsh_env__
-        env_path, bin_path = self[name]
+        ve = self[name]
         if 'VIRTUAL_ENV' in env:
             self.deactivate()
 
         type(self).oldvars = {'PATH': list(env['PATH'])}
-        env['PATH'].insert(0, bin_path)
-        env['VIRTUAL_ENV'] = env_path
+        env['PATH'].insert(0, ve.bin)
+        env['VIRTUAL_ENV'] = ve.env
         if 'PYTHONHOME' in env:
             type(self).oldvars['PYTHONHOME'] = env.pop('PYTHONHOME')
 
+        events.vox_on_activate.fire(name=name)
+
     def deactivate(self):
         """
-        Deactive the active virtual environment. Returns the name of it.
+        Deactivate the active virtual environment. Returns its name.
         """
         env = builtins.__xonsh_env__
         if 'VIRTUAL_ENV' not in env:
             raise NoEnvironmentActive('No environment currently active.')
 
-        env_path, bin_path = self[...]
         env_name = self.active()
 
         if hasattr(type(self), 'oldvars'):
@@ -203,6 +307,7 @@ class Vox(collections.abc.Mapping):
 
         env.pop('VIRTUAL_ENV')
 
+        events.vox_on_deactivate.fire(name=env_name)
         return env_name
 
     def __delitem__(self, name):
@@ -222,3 +327,5 @@ class Vox(collections.abc.Mapping):
             # No current venv, ... fails
             pass
         shutil.rmtree(env_path)
+
+        events.vox_on_delete.fire(name=name)
